/* --COPYRIGHT--,BSD
 * Copyright (c) 2015, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
#include <math.h>
#include <stdbool.h>
#include <stdio.h>
#include <string.h>
#include <stdint.h>

#include "msp430.h"

#include "driverlib.h"

#include "grlib.h"
#include "grlib/Library/grlib.h"
#include "grlib/LcdDriver/kitronix320x240x16_ssd2119_spi.h"

#include "demo.h"
#include "shapes.h"

/*
 * Graphic library context
 */
extern tContext g_sContext;

/*
 * Globals
 */
uint32_t g_Rotated3D[SHAPE_MAX_SIZE][2][3];                    // Rotated 3D points (shared buffer)
struct ui16Vector2D g_ui16Rotated2D[SHAPE_MAX_SIZE][2];        // Rotated 2D fixed point data
struct ui16Vector2D g_ui16Rotated2DPrev[SHAPE_MAX_SIZE][2];    // Previous rotated 2D fixed point data
uint16_t g_ui16Rotated2DPrevLength;                                // Previous rotated 2D fixed point data length

/*
 * Inline _Q15mpy function for faster MPY operation.
 */
static inline _q15 _Q15mpyi(_q15 q15Arg1,
                            _q15 q15Arg2)
{
    MPYS = q15Arg1;
    OP2 = q15Arg2;

    return(RESHI);
}

void demo_updateFrame(uint16_t mode,
                      uint16_t shape)
{
    bool scale;
    uint16_t size;

    /* Use floating point math. */
    if(mode == DEMO_MODE_FLOAT)
    {
        /* Reset and start timer. */
        TIMER_A_clear(TIMER_A1_BASE);
        TIMER_A_startCounter(TIMER_A1_BASE, TIMER_A_UP_MODE);

        /* Display the cube. */
        if(shape == DEMO_SHAPE_CUBE)
        {
            /* Perform the 3D rotation on the cube. */
            demo_fRotate3D(gc_fCube3D, (fVector3D (
                                            *)[2])g_Rotated3D, SHAPE_CUBE_SIZE);
            scale = true;
            size = SHAPE_CUBE_SIZE;
        }
        /* Display the dodecahedron. */
        else if(shape == DEMO_SHAPE_DODECAHEDRON)
        {
            /* Perform the 3D rotation on the dodecahedron. */
            demo_fRotate3D(gc_fDodecahedron3D, (fVector3D (
                                                    *)[2])g_Rotated3D,
                           SHAPE_DODECAHEDRON_SIZE);
            scale = true;
            size = SHAPE_DODECAHEDRON_SIZE;
        }
        /* Display the hypercube. */
        else if(shape == DEMO_SHAPE_HYPERCUBE)
        {
            /* Perform the 4D rotation and projection on the hypercube. */
            demo_fRotate4Dto3D(gc_fHypercube4D, (fVector3D (
                                                     *)[2])g_Rotated3D,
                               SHAPE_HYPERCUBE_SIZE);
            scale = false;
            size = SHAPE_HYPERCUBE_SIZE;
        }

        /* Convert the 3D cube to a 2D image. */
        demo_fConvert3Dto2D((fVector3D (
                                 *)[2])g_Rotated3D, g_ui16Rotated2D, size,
                            scale);

        /* Stop timer and add to total graphics count. */
        TIMER_A_stop(TIMER_A1_BASE);
        g_i32GraphicsCycles += TIMER_A_getCounterValue(TIMER_A1_BASE);

        /* Reset and start timer. */
        TIMER_A_clear(TIMER_A1_BASE);
        TIMER_A_startCounter(TIMER_A1_BASE, TIMER_A_UP_MODE);

        /* Draw the cube. */
        demo_ui16Draw2D(g_ui16Rotated2D, size);

        /* Stop timer and add to total display count. */
        TIMER_A_stop(TIMER_A1_BASE);
        g_i32DisplayCycles += TIMER_A_getCounterValue(TIMER_A1_BASE);
    }
    /* Use fixed point IQ math. */
    else if(mode == DEMO_MODE_FIXED)
    {
        /* Reset and start timer. */
        TIMER_A_clear(TIMER_A1_BASE);
        TIMER_A_startCounter(TIMER_A1_BASE, TIMER_A_UP_MODE);

        /* Display the cube. */
        if(shape == DEMO_SHAPE_CUBE)
        {
            /* Perform the 3D rotation on the cube. */
            demo_qRotate3D(gc_qCube3D, (qVector3D (
                                            *)[2])g_Rotated3D, SHAPE_CUBE_SIZE);
            scale = true;
            size = SHAPE_CUBE_SIZE;
        }
        /* Display the dodecahedron. */
        else if(shape == DEMO_SHAPE_DODECAHEDRON)
        {
            /* Perform the 3D rotation on the dodecahedron. */
            demo_qRotate3D(gc_qDodecahedron3D, (qVector3D (
                                                    *)[2])g_Rotated3D,
                           SHAPE_DODECAHEDRON_SIZE);
            scale = true;
            size = SHAPE_DODECAHEDRON_SIZE;
        }
        /* Display the hypercube. */
        else if(shape == DEMO_SHAPE_HYPERCUBE)
        {
            /* Perform the 4D rotation and projection on the hypercube. */
            demo_qRotate4Dto3D(gc_qHypercube4D, (qVector3D (
                                                     *)[2])g_Rotated3D,
                               SHAPE_HYPERCUBE_SIZE);
            scale = false;
            size = SHAPE_HYPERCUBE_SIZE;
        }

        /* Convert the 3D dodecahedron to a 2D image. */
        demo_qConvert3Dto2D((qVector3D (
                                 *)[2])g_Rotated3D, g_ui16Rotated2D, size,
                            scale);

        /* Stop timer and add to total graphics count. */
        TIMER_A_stop(TIMER_A1_BASE);
        g_i32GraphicsCycles += TIMER_A_getCounterValue(TIMER_A1_BASE);

        /* Reset and start timer. */
        TIMER_A_clear(TIMER_A1_BASE);
        TIMER_A_startCounter(TIMER_A1_BASE, TIMER_A_UP_MODE);

        /* Draw the hypercube. */
        demo_ui16Draw2D(g_ui16Rotated2D, size);

        /* Stop timer and add to total display count. */
        TIMER_A_stop(TIMER_A1_BASE);
        g_i32DisplayCycles += TIMER_A_getCounterValue(TIMER_A1_BASE);
    }
}

void demo_ui16Draw2D(ui16Vector2D shape[][2],
                     uint16_t length)
{
    uint16_t i;
    uint32_t lineColor;

    /* Get the next color in the rotation. */
    lineColor = demo_getNextColor();

    /* Clear additional lines from a different length shape. */
    GrContextForegroundSet(&g_sContext, ClrBlack);
    for(i = length; i < g_ui16Rotated2DPrevLength; i++)
    {
        /* Clear the previous line. */
        GrLineDraw(&g_sContext,
                   g_ui16Rotated2DPrev[i][0].x,
                   g_ui16Rotated2DPrev[i][0].y,
                   g_ui16Rotated2DPrev[i][1].x,
                   g_ui16Rotated2DPrev[i][1].y);
    }

    /* Clear the previous line and draw the new line. */
    for(i = 0; i < length; i++)
    {
        /* Clear the previous line. */
        GrContextForegroundSet(&g_sContext, ClrBlack);
        GrLineDraw(&g_sContext,
                   g_ui16Rotated2DPrev[i][0].x,
                   g_ui16Rotated2DPrev[i][0].y,
                   g_ui16Rotated2DPrev[i][1].x,
                   g_ui16Rotated2DPrev[i][1].y);

        /* Save the new line to the previous. */
        g_ui16Rotated2DPrev[i][0] = shape[i][0];
        g_ui16Rotated2DPrev[i][1] = shape[i][1];

        /* Draw the new line. */
        GrContextForegroundSet(&g_sContext, lineColor);
        GrLineDraw(&g_sContext, shape[i][0].x, shape[i][0].y, shape[i][1].x,
                   shape[i][1].y);
    }

    /* Save the new previous length. */
    g_ui16Rotated2DPrevLength = length;
}

void demo_fConvert3Dto2D(fVector3D shape3D[][2],
                         ui16Vector2D shape2D[][2],
                         uint16_t length,
                         bool scale)
{
    uint16_t i;
    uint16_t x0, y0, x1, y1;

    static uint16_t ui16Direction = DEMO_DIRECTION_UP;
    static float fDemoSize = DEMO_SIZE_MIN;

    /* Update the size. */
    if(ui16Direction == DEMO_DIRECTION_UP)
    {
        fDemoSize += DEMO_SIZE_SPEED;
        if(fDemoSize > DEMO_SIZE_MAX)
        {
            fDemoSize = DEMO_SIZE_MAX;
            ui16Direction = DEMO_DIRECTION_DOWN;
        }
    }
    else
    {
        fDemoSize -= DEMO_SIZE_SPEED;
        if(fDemoSize < DEMO_SIZE_MIN)
        {
            fDemoSize = DEMO_SIZE_MIN;
            ui16Direction = DEMO_DIRECTION_UP;
        }
    }

    for(i = 0; i < length; i++)
    {
        if(scale)
        {
            /* Get rotated coordinate for start of line. */
            x0 = (int16_t)(shape3D[i][0].x * fDemoSize) + DEMO_XOFFSET;
            y0 = (int16_t)(shape3D[i][0].y * fDemoSize) + DEMO_YOFFSET;

            /* Get rotated coordinate for end of line. */
            x1 = (int16_t)(shape3D[i][1].x * fDemoSize) + DEMO_XOFFSET;
            y1 = (int16_t)(shape3D[i][1].y * fDemoSize) + DEMO_YOFFSET;
        }
        else
        {
            /* Get rotated coordinate for start of line. */
            x0 = (int16_t)(shape3D[i][0].x * DEMO_SIZE_MAX) + DEMO_XOFFSET;
            y0 = (int16_t)(shape3D[i][0].y * DEMO_SIZE_MAX) + DEMO_YOFFSET;

            /* Get rotated coordinate for end of line. */
            x1 = (int16_t)(shape3D[i][1].x * DEMO_SIZE_MAX) + DEMO_XOFFSET;
            y1 = (int16_t)(shape3D[i][1].y * DEMO_SIZE_MAX) + DEMO_YOFFSET;
        }

        /* make sure x0, x1 isnt greater than LCD_X_SIZE or less than 0. */
        x0 = _Qsat(x0, (LCD_HORIZONTAL_MAX - 1), 1);
        x1 = _Qsat(x1, (LCD_HORIZONTAL_MAX - 1), 1);

        /* make sure y0, y1 isnt greater than LCD_Y_SIZE or less than 0. */
        y0 = _Qsat(y0, (LCD_VERTICAL_MAX - 1), 1);
        y1 = _Qsat(y1, (LCD_VERTICAL_MAX - 1), 1);

        shape2D[i][0].x = x0;
        shape2D[i][0].y = y0;
        shape2D[i][1].x = x1;
        shape2D[i][1].y = y1;
    }
}

void demo_qConvert3Dto2D(qVector3D shape3D[][2],
                         ui16Vector2D shape2D[][2],
                         uint16_t length,
                         bool scale)
{
    uint16_t i;
    uint16_t x0, y0, x1, y1;

    static uint16_t ui16Direction = DEMO_DIRECTION_UP;
    static uint16_t q8DemoSize = _Q8(DEMO_SIZE_MIN);

    /* Update the size. */
    if(ui16Direction == DEMO_DIRECTION_UP)
    {
        q8DemoSize += _Q8(DEMO_SIZE_SPEED);
        if(q8DemoSize > _Q8(DEMO_SIZE_MAX))
        {
            q8DemoSize = _Q8(DEMO_SIZE_MAX);
            ui16Direction = DEMO_DIRECTION_DOWN;
        }
    }
    else
    {
        q8DemoSize -= _Q8(DEMO_SIZE_SPEED);
        if(q8DemoSize < _Q8(DEMO_SIZE_MIN))
        {
            q8DemoSize = _Q8(DEMO_SIZE_MIN);
            ui16Direction = DEMO_DIRECTION_UP;
        }
    }

    for(i = 0; i < length; i++)
    {
        if(scale)
        {
            /* Scale the rotated points and get the coordinate for start of the line. */
            x0 =
                (uint16_t)((_IQ8mpy(shape3D[i][0].x,
                                    q8DemoSize) +
                            _Q(0.5)) >> GLOBAL_Q) + DEMO_XOFFSET;
            y0 =
                (uint16_t)((_IQ8mpy(shape3D[i][0].y,
                                    q8DemoSize) +
                            _Q(0.5)) >> GLOBAL_Q) + DEMO_YOFFSET;

            /* Scale the rotated points and get the coordinate for end of the line. */
            x1 =
                (uint16_t)((_IQ8mpy(shape3D[i][1].x,
                                    q8DemoSize) +
                            _Q(0.5)) >> GLOBAL_Q) + DEMO_XOFFSET;
            y1 =
                (uint16_t)((_IQ8mpy(shape3D[i][1].y,
                                    q8DemoSize) +
                            _Q(0.5)) >> GLOBAL_Q) + DEMO_YOFFSET;
        }
        else
        {
            /* Scale the rotated points and get the coordinate for start of the line. */
            x0 =
                (uint16_t)((_IQ8mpy(shape3D[i][0].x,
                                    _Q8(DEMO_SIZE_MAX)) +
                            _Q(0.5)) >> GLOBAL_Q) + DEMO_XOFFSET;
            y0 =
                (uint16_t)((_IQ8mpy(shape3D[i][0].y,
                                    _Q8(DEMO_SIZE_MAX)) +
                            _Q(0.5)) >> GLOBAL_Q) + DEMO_YOFFSET;

            /* Scale the rotated points and get the coordinate for end of the line. */
            x1 =
                (uint16_t)((_IQ8mpy(shape3D[i][1].x,
                                    _Q8(DEMO_SIZE_MAX)) +
                            _Q(0.5)) >> GLOBAL_Q) + DEMO_XOFFSET;
            y1 =
                (uint16_t)((_IQ8mpy(shape3D[i][1].y,
                                    _Q8(DEMO_SIZE_MAX)) +
                            _Q(0.5)) >> GLOBAL_Q) + DEMO_YOFFSET;
        }

        /* make sure x0, x1 isnt greater than LCD_X_SIZE or less than 0. */
        x0 = _Qsat(x0, (LCD_HORIZONTAL_MAX - 1), 1);
        x1 = _Qsat(x1, (LCD_HORIZONTAL_MAX - 1), 1);

        /* make sure y0, y1 isnt greater than LCD_Y_SIZE or less than 0. */
        y0 = _Qsat(y0, (LCD_VERTICAL_MAX - 1), 1);
        y1 = _Qsat(y1, (LCD_VERTICAL_MAX - 1), 1);

        shape2D[i][0].x = x0;
        shape2D[i][0].y = y0;
        shape2D[i][1].x = x1;
        shape2D[i][1].y = y1;
    }
}

void demo_fRotate3D(const fVector3D shape[][2],
                    fVector3D rotated[][2],
                    uint16_t length)
{
    uint16_t i, j;                // loop counters
    fVector3D *pPt;               // pointer to the rotated point
    float fXTemp, fYTemp;         // temporary x and y variables
    float fCosX, fCosY, fCosZ;    // temporary cos variables
    float fSinX, fSinY, fSinZ;    // temporary sin variables

    static float fAngleX = TWO_PI / 4;
    static float fAngleY = TWO_PI / 8;
    static float fAngleZ = TWO_PI / 2;

    /* Update angles */
    if((fAngleX += DEMO_ROTATION_SPEED) > TWO_PI)
    {
        fAngleX = 0;
    }
    if((fAngleY += DEMO_ROTATION_SPEED) > TWO_PI)
    {
        fAngleY = 0;
    }
    if((fAngleZ += DEMO_ROTATION_SPEED) > TWO_PI)
    {
        fAngleZ = 0;
    }

    /* Compute the cos and sin values. */
    fCosX = cosf(fAngleX);
    fCosY = cosf(fAngleY);
    fCosZ = cosf(fAngleZ);
    fSinX = sinf(fAngleX);
    fSinY = sinf(fAngleY);
    fSinZ = sinf(fAngleZ);

    /* Rotate the 3D points. */
    for(i = 0; i < length; i++)
    {
        for(j = 0; j < 2; j++)
        {
            /* Copy the 3D point to rotate. */
            rotated[i][j] = shape[i][j];
            pPt = &rotated[i][j];

            /* Rotate around Z */
            fXTemp = (pPt->x * fCosZ) - (pPt->y * fSinZ);
            pPt->y = (pPt->x * fSinZ) + (pPt->y * fCosZ);
            pPt->x = fXTemp;

            /* Rotate around Y */
            fXTemp = (pPt->x * fCosY) + (pPt->z * fSinY);
            pPt->z = -(pPt->x * fSinY) + (pPt->z * fCosY);
            pPt->x = fXTemp;

            /* Rotate around X */
            fYTemp = (pPt->y * fCosX) - (pPt->z * fSinX);
            pPt->z = (pPt->y * fSinX) + (pPt->z * fCosX);
            pPt->y = fYTemp;
        }
    }
}

void demo_qRotate3D(const qVector3D shape[][2],
                    qVector3D rotated[][2],
                    uint16_t length)
{
    uint16_t i, j;                    // loop counters
    qVector3D *pPt;                 // pointer to the rotated point
    _q qXTemp, qYTemp;                // temporary x and y variables
    _q15 q15CosX, q15CosY, q15CosZ;    // temporary cos variables
    _q15 q15SinX, q15SinY, q15SinZ;    // temporary sin variables

    static _q12 q12AngleX = _Q12(TWO_PI / 4);
    static _q12 q12AngleY = _Q12(TWO_PI / 8);
    static _q12 q12AngleZ = _Q12(TWO_PI / 2);

    /* Update angles */
    if((q12AngleX += _Q12(DEMO_ROTATION_SPEED)) > _Q12(TWO_PI))
    {
        q12AngleX = 0;
    }
    if((q12AngleY += _Q12(DEMO_ROTATION_SPEED)) > _Q12(TWO_PI))
    {
        q12AngleY = 0;
    }
    if((q12AngleZ += _Q12(DEMO_ROTATION_SPEED)) > _Q12(TWO_PI))
    {
        q12AngleZ = 0;
    }

    /* Compute the cos and sin values in q15 format with saturation. */
    q15CosX =
        _Qsat(_Q12cos(q12AngleX), (MAX_Q_POS >> 3), (MAX_Q_NEG >> 3)) << 3;
    q15CosY =
        _Qsat(_Q12cos(q12AngleY), (MAX_Q_POS >> 3), (MAX_Q_NEG >> 3)) << 3;
    q15CosZ =
        _Qsat(_Q12cos(q12AngleZ), (MAX_Q_POS >> 3), (MAX_Q_NEG >> 3)) << 3;
    q15SinX =
        _Qsat(_Q12sin(q12AngleX), (MAX_Q_POS >> 3), (MAX_Q_NEG >> 3)) << 3;
    q15SinY =
        _Qsat(_Q12sin(q12AngleY), (MAX_Q_POS >> 3), (MAX_Q_NEG >> 3)) << 3;
    q15SinZ =
        _Qsat(_Q12sin(q12AngleZ), (MAX_Q_POS >> 3), (MAX_Q_NEG >> 3)) << 3;

    /* Enable fractional mode once when using inlined multiplication. */
    MPY32_enableFractionalMode();

    /* Rotate the 3D points. */
    for(i = 0; i < length; i++)
    {
        for(j = 0; j < 2; j++)
        {
            /* Copy the 3D point to rotate. */
            rotated[i][j] = shape[i][j];
            pPt = &rotated[i][j];

            /* Rotate around Z */
            qXTemp = _Q15mpyi(pPt->x, q15CosZ) - _Q15mpyi(pPt->y, q15SinZ);
            pPt->y = _Q15mpyi(pPt->x, q15SinZ) + _Q15mpyi(pPt->y, q15CosZ);
            pPt->x = qXTemp;

            /* Rotate around Y */
            qXTemp = _Q15mpyi(pPt->x, q15CosY) + _Q15mpyi(pPt->z, q15SinY);
            pPt->z = -_Q15mpyi(pPt->x, q15SinY) + _Q15mpyi(pPt->z, q15CosY);
            pPt->x = qXTemp;

            /* Rotate around X */
            qYTemp = _Q15mpyi(pPt->y, q15CosX) - _Q15mpyi(pPt->z, q15SinX);
            pPt->z = _Q15mpyi(pPt->y, q15SinX) + _Q15mpyi(pPt->z, q15CosX);
            pPt->y = qYTemp;
        }
    }

    /* Disable fractional mode before returning. */
    MPY32_disableFractionalMode();
}

void demo_fRotate4Dto3D(const fVector4D shape[][2],
                        fVector3D rotated[][2],
                        uint16_t length)
{
    uint16_t i, j;                            // loop counters
    const fVector4D *pPt;                     // pointer to the point to rotate
    float fXTemp, fYTemp, fZTemp, fWTemp;    // temporary x and y variables
    float fCosY, fCosXY, fCosZW;            // temporary cos variables
    float fSinY, fSinXY, fSinZW;            // temporary sin variables

    static float fAngleXY = TWO_PI / 8;
    static float fAngleZW = TWO_PI / 4;

    /* Update angles */
    if((fAngleXY += DEMO_ROTATION_SPEED) > TWO_PI)
    {
        fAngleXY = 0;
    }
    if((fAngleZW += DEMO_ROTATION_SPEED) > TWO_PI)
    {
        fAngleZW = 0;
    }

    /* Compute the cos and sin values. */
    fCosY = cosf(-TWO_PI / 8);
    fCosXY = cosf(fAngleXY);
    fCosZW = cosf(fAngleZW);
    fSinY = sinf(-TWO_PI / 8);
    fSinXY = sinf(fAngleXY);
    fSinZW = sinf(fAngleZW);

    /* Rotate the 4D points. */
    for(i = 0; i < length; i++)
    {
        for(j = 0; j < 2; j++)
        {
            /* Set the local pointer. */
            pPt = &shape[i][j];

            /* Rotate around XY plane. */
            fZTemp = (pPt->z * fCosXY) - (pPt->w * fSinXY);
            fWTemp = (pPt->z * fSinXY) + (pPt->w * fCosXY);

            /* Rotate around ZW plane. */
            fXTemp = (pPt->x * fCosZW) - (pPt->y * fSinZW);
            fYTemp = (pPt->x * fSinZW) + (pPt->y * fCosZW);

            /* Translate W so that it is always positive. */
            fWTemp += 3;

            /* Project X, Y and Z to 3D. */
            fXTemp = (fXTemp / fWTemp) * 2;
            fYTemp = (fYTemp / fWTemp) * 2;
            fZTemp = (fZTemp / fWTemp) * 2;

            /* Rotate around Y to get a better perspective and store the rotated points. */
            rotated[i][j].x = (fXTemp * fCosY) + (fZTemp * fSinY);
            rotated[i][j].z = -(fXTemp * fSinY) + (fZTemp * fCosY);
            rotated[i][j].y = fYTemp;
        }
    }
}

void demo_qRotate4Dto3D(const qVector4D shape[][2],
                        qVector3D rotated[][2],
                        uint16_t length)
{
    uint16_t i, j;                        // loop counters
    const qVector4D *pPt;                 // pointer to the point to rotate
    _q qXTemp, qYTemp, qZTemp, qWTemp;    // temporary x and y variables
    _q15 q15CosY, q15CosXY, q15CosZW;    // temporary cos variables
    _q15 q15SinY, q15SinXY, q15SinZW;    // temporary sin variables

    static _q12 q12AngleXY = _Q12(TWO_PI / 8);
    static _q12 q12AngleZW = _Q12(TWO_PI / 4);

    /* Update angles */
    if((q12AngleXY += _Q12(DEMO_ROTATION_SPEED)) > _Q12(TWO_PI))
    {
        q12AngleXY = 0;
    }
    if((q12AngleZW += _Q12(DEMO_ROTATION_SPEED)) > _Q12(TWO_PI))
    {
        q12AngleZW = 0;
    }

    /* Compute the cos and sin values in q15 format with saturation. */
    q15CosY =
        _Qsat(_Q12cos(_Q12(-TWO_PI / 8)), (MAX_Q_POS >> 3),
              (MAX_Q_NEG >> 3)) << 3;
    q15CosXY =
        _Qsat(_Q12cos(q12AngleXY), (MAX_Q_POS >> 3), (MAX_Q_NEG >> 3)) << 3;
    q15CosZW =
        _Qsat(_Q12cos(q12AngleZW), (MAX_Q_POS >> 3), (MAX_Q_NEG >> 3)) << 3;
    q15SinY =
        _Qsat(_Q12sin(_Q12(-TWO_PI / 8)), (MAX_Q_POS >> 3),
              (MAX_Q_NEG >> 3)) << 3;
    q15SinXY =
        _Qsat(_Q12sin(q12AngleXY), (MAX_Q_POS >> 3), (MAX_Q_NEG >> 3)) << 3;
    q15SinZW =
        _Qsat(_Q12sin(q12AngleZW), (MAX_Q_POS >> 3), (MAX_Q_NEG >> 3)) << 3;

    /* Enable fractional mode once when using inlined multiplication. */
    MPY32_enableFractionalMode();

    /* Rotate the 4D points. */
    for(i = 0; i < length; i++)
    {
        for(j = 0; j < 2; j++)
        {
            /* Set the local pointer. */
            pPt = &shape[i][j];

            /* Rotate around XY plane. */
            qZTemp = _Q15mpyi(pPt->z, q15CosXY) - _Q15mpyi(pPt->w, q15SinXY);
            qWTemp = _Q15mpyi(pPt->z, q15SinXY) + _Q15mpyi(pPt->w, q15CosXY);

            /* Rotate around ZW plane. */
            qXTemp = _Q15mpyi(pPt->x, q15CosZW) - _Q15mpyi(pPt->y, q15SinZW);
            qYTemp = _Q15mpyi(pPt->x, q15SinZW) + _Q15mpyi(pPt->y, q15CosZW);

            /* Translate W so that it is always positive. */
            qWTemp += _Q(3);

            /* Project X, Y and Z to 3D. */
            qXTemp = _Qmpy2(_Qdiv(qXTemp, qWTemp));
            qYTemp = _Qmpy2(_Qdiv(qYTemp, qWTemp));
            qZTemp = _Qmpy2(_Qdiv(qZTemp, qWTemp));

            /* Rotate around Y to get a better perspective and store the rotated points. */
            rotated[i][j].x = _Q15mpyi(qXTemp, q15CosY) + _Q15mpyi(qZTemp,
                                                                   q15SinY);
            rotated[i][j].z = -_Q15mpyi(qXTemp, q15SinY) + _Q15mpyi(qZTemp,
                                                                    q15CosY);
            rotated[i][j].y = qYTemp;
        }
    }

    /* Disable fractional mode before returning. */
    MPY32_disableFractionalMode();
}

int32_t demo_getNextColor(void)
{
    int16_t i16ColorChannel;
    static uint8_t ui8Index;
    static uint8_t ui8Direction;
    static uint32_t ui32Color = 0xffffffff;

    /* Initialize color, index and direction the first time this is called. */
    if(ui32Color == 0xffffffff)
    {
        ui8Index = DEMO_INDEX_GREEN;
        ui8Direction = DEMO_DIRECTION_UP;
        ui32Color = 0x00ff0000;
    }
    /* Calculate the next color. */
    else
    {
        /* Increment current index. */
        if(ui8Direction == DEMO_DIRECTION_UP)
        {
            /* Get the current color channel based on 8-bit index. */
            i16ColorChannel = ((uint8_t *)&ui32Color)[ui8Index];

            /* Increment the color channel. */
            i16ColorChannel += DEMO_COLOR_SPEED;

            /* Check if the current value has reached the maximum allowed value. */
            if(i16ColorChannel >= 0xff)
            {
                /* Save the saturated value back to the current color. */
                ((uint8_t *)&ui32Color)[ui8Index] = (uint8_t)0xff;

                /* Set the new direction and index. */
                (ui8Index == DEMO_INDEX_RED) ? (ui8Index =
                                                    DEMO_INDEX_BLUE) : (ui8Index
                                                                        ==
                                                                        DEMO_INDEX_GREEN
                                                                        ? (
                                                                            ui8Index
                = DEMO_INDEX_RED) : (ui8Index =
                                                                                                              DEMO_INDEX_GREEN));
                ui8Direction = DEMO_DIRECTION_DOWN;

                /* Calculate the remainder after saturation and subtract from the next index. */
                i16ColorChannel -= 0xff;
                ((uint8_t *)&ui32Color)[ui8Index] -= i16ColorChannel;
            }
            else
            {
                /* Save the color channel back to the current color. */
                ((uint8_t *)&ui32Color)[ui8Index] = (uint8_t)i16ColorChannel;
            }
        }
        /* Decrement current index. */
        else
        {
            /* Get the current color channel based on 8-bit index. */
            i16ColorChannel = ((uint8_t *)&ui32Color)[ui8Index];

            /* Decrement the color channel. */
            i16ColorChannel -= DEMO_COLOR_SPEED;

            /* Check if the current value has reached the minimum allowed value. */
            if(i16ColorChannel <= 0x00)
            {
                /* Save the saturated value back to the current color. */
                ((uint8_t *)&ui32Color)[ui8Index] = (uint8_t)0x00;

                /* Set the new direction and index. */
                (ui8Index == DEMO_INDEX_RED) ? (ui8Index =
                                                    DEMO_INDEX_BLUE) : (ui8Index
                                                                        ==
                                                                        DEMO_INDEX_GREEN
                                                                        ? (
                                                                            ui8Index
                = DEMO_INDEX_RED) : (ui8Index =
                                                                                                              DEMO_INDEX_GREEN));
                ui8Direction = DEMO_DIRECTION_UP;

                /* Add the remaining difference from the next index. */
                ((uint8_t *)&ui32Color)[ui8Index] += i16ColorChannel;
            }
            else
            {
                /* Save the color channel back to the current color. */
                ((uint8_t *)&ui32Color)[ui8Index] = (uint8_t)i16ColorChannel;
            }
        }
    }

    /* Return the 24-bit color. */
    return(ui32Color);
}
