/* --COPYRIGHT--,BSD
 * Copyright (c) 2015, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
#include <stdbool.h>
#include <stdint.h>

#include <msp430.h>

#include "driverlib.h"

#include "../analog.h"
#include "../touch.h"

#if defined(__MSP430_HAS_ADC12_B__)

/*
 * Initialize the analog hardware to read from the touch screen.
 */
void analog_init(void)
{
    /* Initialize the ADC12_B Module. */
    ADC12_B_init(ADC12_B_BASE,
                 ADC12_B_SAMPLEHOLDSOURCE_SC,
                 ADC12_B_CLOCKSOURCE_ADC12OSC,
                 ADC12_B_CLOCKDIVIDER_1,
                 ADC12_B_CLOCKPREDIVIDER__1,
                 ADC12_B_NOINTCH);
    ADC12_B_setupSamplingTimer(ADC12_B_BASE,
                               ADC12_B_CYCLEHOLD_64_CYCLES,
                               ADC12_B_CYCLEHOLD_64_CYCLES,
                               ADC12_B_MULTIPLESAMPLESDISABLE);
    ADC12_B_enable(ADC12_B_BASE);

    /* Configure Y+ input to memory buffer. */
    ADC12_B_memoryConfigure(ADC12_B_BASE,
                            TOUCH_Y_PLUS_MEMORY,
                            TOUCH_Y_PLUS_INPUT,
                            ADC12_B_VREFPOS_AVCC_VREFNEG_VSS,
                            ADC12_B_NOTENDOFSEQUENCE,
                            ADC12_B_WINDOW_COMPARATOR_DISABLE,
                            ADC12_B_DIFFERENTIAL_MODE_DISABLE);

    /* Configure X+ input to memory buffer. */
    ADC12_B_memoryConfigure(ADC12_B_BASE,
                            TOUCH_X_PLUS_MEMORY,
                            TOUCH_X_PLUS_INPUT,
                            ADC12_B_VREFPOS_AVCC_VREFNEG_VSS,
                            ADC12_B_NOTENDOFSEQUENCE,
                            ADC12_B_WINDOW_COMPARATOR_DISABLE,
                            ADC12_B_DIFFERENTIAL_MODE_DISABLE);
}

/*
 * Returns true when a touch is detected.
 */
bool analog_detectedTouch(void)
{
    uint16_t ui16ADCtemp;

    /* Set X- and Y- as output and Y+ as input (floating). */
    GPIO_setAsOutputPin(TOUCH_X_MINUS_PORT, TOUCH_X_MINUS_PIN);
    GPIO_setAsOutputPin(TOUCH_Y_MINUS_PORT, TOUCH_Y_MINUS_PIN);
    GPIO_setAsInputPin(TOUCH_Y_PLUS_PORT, TOUCH_Y_PLUS_PIN);

    /* Set X- high and Y- low to detect touch. */
    GPIO_setOutputHighOnPin(TOUCH_X_MINUS_PORT, TOUCH_X_MINUS_PIN);
    GPIO_setOutputLowOnPin(TOUCH_Y_MINUS_PORT, TOUCH_Y_MINUS_PIN);

    /* Sample the X+ ADC channel to check if there is currently a touch. */
    GPIO_setAsPeripheralModuleFunctionInputPin(TOUCH_X_PLUS_PORT,
                                               TOUCH_X_PLUS_PIN,
                                               GPIO_TERNARY_MODULE_FUNCTION);
    ADC12_B_clearInterrupt(ADC12_B_BASE, TOUCH_X_PLUS_REG, TOUCH_X_PLUS_IFG);
    ADC12_B_startConversion(ADC12_B_BASE, TOUCH_X_PLUS_MEMORY >> 1,
                            ADC12_B_SINGLECHANNEL);
    while(!ADC12_B_getInterruptStatus(ADC12_B_BASE, TOUCH_X_PLUS_REG,
                                      TOUCH_X_PLUS_IFG))
    {
        ;
    }
    ui16ADCtemp = ADC12_B_getResults(ADC12_B_BASE, TOUCH_X_PLUS_MEMORY);

    /* Check if the detected touch is below the threshold. */
    if(ui16ADCtemp < TOUCH_THRESHOLD)
    {
        return(true);
    }
    else
    {
        return(false);
    }
}

/*
 * Sample the X analog axis.
 */
uint16_t analog_sampleX(void)
{
    /* Set X+ and X- as output and Y- as input (floating). */
    GPIO_setAsOutputPin(TOUCH_X_PLUS_PORT, TOUCH_X_PLUS_PIN);
    GPIO_setAsOutputPin(TOUCH_X_MINUS_PORT, TOUCH_X_MINUS_PIN);
    GPIO_setAsInputPin(TOUCH_Y_MINUS_PORT, TOUCH_Y_MINUS_PIN);

    /* Set X+ high and X- low. */
    GPIO_setOutputHighOnPin(TOUCH_X_PLUS_PORT, TOUCH_X_PLUS_PIN);
    GPIO_setOutputLowOnPin(TOUCH_X_MINUS_PORT, TOUCH_X_MINUS_PIN);

    /* Sample the Y+ ADC channel. */
    GPIO_setAsPeripheralModuleFunctionOutputPin(TOUCH_Y_PLUS_PORT,
                                                TOUCH_Y_PLUS_PIN,
                                                GPIO_TERNARY_MODULE_FUNCTION);
    ADC12_B_clearInterrupt(ADC12_B_BASE, TOUCH_Y_PLUS_REG, TOUCH_Y_PLUS_IFG);
    ADC12_B_startConversion(ADC12_B_BASE, TOUCH_Y_PLUS_MEMORY >> 1,
                            ADC12_B_SINGLECHANNEL);
    while(!ADC12_B_getInterruptStatus(ADC12_B_BASE, TOUCH_Y_PLUS_REG,
                                      TOUCH_Y_PLUS_IFG))
    {
        ;
    }

    /* Return the anlog result. */
    return(ADC12_B_getResults(ADC12_B_BASE, TOUCH_Y_PLUS_MEMORY));
}

/*
 * Sample the Y analog axis.
 */
uint16_t analog_sampleY(void)
{
    /* Set Y+ and Y- as output and X- as input (floating). */
    GPIO_setAsOutputPin(TOUCH_Y_PLUS_PORT, TOUCH_Y_PLUS_PIN);
    GPIO_setAsOutputPin(TOUCH_Y_MINUS_PORT, TOUCH_Y_MINUS_PIN);
    GPIO_setAsInputPin(TOUCH_X_MINUS_PORT, TOUCH_X_MINUS_PIN);

    /* Set Y+ low and Y- high. */
    GPIO_setOutputLowOnPin(TOUCH_Y_PLUS_PORT, TOUCH_Y_PLUS_PIN);
    GPIO_setOutputHighOnPin(TOUCH_Y_MINUS_PORT, TOUCH_Y_MINUS_PIN);

    /* Sample the X+ ADC channel. */
    GPIO_setAsPeripheralModuleFunctionOutputPin(TOUCH_X_PLUS_PORT,
                                                TOUCH_X_PLUS_PIN,
                                                GPIO_TERNARY_MODULE_FUNCTION);
    ADC12_B_clearInterrupt(ADC12_B_BASE, TOUCH_X_PLUS_REG, TOUCH_X_PLUS_IFG);
    ADC12_B_startConversion(ADC12_B_BASE, TOUCH_X_PLUS_MEMORY >> 1,
                            ADC12_B_SINGLECHANNEL);
    while(!ADC12_B_getInterruptStatus(ADC12_B_BASE, TOUCH_X_PLUS_REG,
                                      TOUCH_X_PLUS_IFG))
    {
        ;
    }

    /* Return the anlog result. */
    return(ADC12_B_getResults(ADC12_B_BASE, TOUCH_X_PLUS_MEMORY));
}

#endif
