/* --COPYRIGHT--,BSD
 * Copyright (c) 2015, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
#include <stdint.h>

#include <msp430.h>

#include "driverlib.h"

#include "grlib/Library/grlib.h"
#include "grlib/LcdDriver/kitronix320x240x16_ssd2119_spi.h"

#include "button.h"
#include "demo.h"
#include "touch.h"

/*
 * MCLK frequency to run the demo at.
 */
#define MCLK_FREQUENCY    8000000

/*
 * Graphic library context
 */
tContext g_sContext;

/*
 * Touch screen context
 */
touch_context g_sTouchContext;

/*
 * Globals
 */
uint16_t g_ui16MathMode;
uint16_t g_ui16Shape;
uint32_t g_i32GraphicsCycles;
uint32_t g_i32DisplayCycles;

/*
 * Prototypes
 */
void boardInit();
void clockInit(void);
void timerInit();

/*
 * Rectangle used to clear the display times.
 */
tRectangle clearDisplayTimes = {112, 204, 148, 240};

/*
 * Strings used to update display times
 */
const int8_t cGraphicsString[] = "Graphic calc time: 00.00 ms";
const int8_t cDisplayString[] = "Display draw time: 00.00 ms";
const int8_t cTotalString[] = "Total frame time:  00.00 ms";

/*
 * Shared display buffer.
 */
int8_t cDisplayBuffer[8];

void main(void)
{
    uint16_t ui16FrameCount;
    _iq20 iq20GraphicsTime;
    _iq20 iq20DisplayTime;

    /* Initialize the demo. */
    boardInit();
    clockInit();
    timerInit();

    /* Globally enable interrupts. */
    __enable_interrupt();

    /* Initialize up the LCD. */
    Kitronix320x240x16_SSD2119Init();
    GrContextInit(&g_sContext, &g_sKitronix320x240x16_SSD2119);
    GrContextForegroundSet(&g_sContext, ClrWhite);
    GrContextBackgroundSet(&g_sContext, ClrBlack);
    GrContextFontSet(&g_sContext, &g_sFontCmss24);

    /* Initialize the resistive touch screen. */
    touch_initInterface();

    /* Initialize the graphical user interface. */
    GrContextForegroundSet(&g_sContext, ClrRed);
    GrStringDraw(&g_sContext, "MSP430 Accelerated Math Demo",
                 AUTO_STRING_LENGTH, 2, 1, TRANSPARENT_TEXT);
    button_drawButton(&buttonMathModeFloatBoarder, ClrDarkSlateGray,
                      &buttonMathModeFloatFill, ClrRed, "Float");
    button_drawButton(&buttonMathModeIQmathBoarder, ClrDarkSlateGray,
                      &buttonMathModeIQmathFill, ClrDarkRed, "IQmathLib");
    button_drawButton(&buttonShapeCubeBoarder, ClrDarkSlateGray,
                      &buttonShapeCubeFill, ClrBlue, "Cube");
    button_drawButton(&buttonShapeDodecahedronBoarder, ClrDarkSlateGray,
                      &buttonShapeDodecahedronFill, ClrDarkBlue,
                      "Dodecahedron");
    button_drawButton(&buttonShapeHypercubeBoarder, ClrDarkSlateGray,
                      &buttonShapeHypercubeFill, ClrDarkBlue, "Hypercube");
    GrContextFontSet(&g_sContext, &g_sFontFixed6x8);
    GrStringDraw(&g_sContext, cGraphicsString, AUTO_STRING_LENGTH, 4, 204,
                 TRANSPARENT_TEXT);
    GrStringDraw(&g_sContext, cDisplayString, AUTO_STRING_LENGTH, 4, 216,
                 TRANSPARENT_TEXT);
    GrStringDraw(&g_sContext, cTotalString, AUTO_STRING_LENGTH, 4, 228,
                 TRANSPARENT_TEXT);
    GrContextFontSet(&g_sContext, &g_sFontCmss24);

    /* Initialize the frame count, math mode and shape before begining the demo. */
    ui16FrameCount = 0;
    g_ui16MathMode = DEMO_MODE_FLOAT;
    g_ui16Shape = DEMO_SHAPE_CUBE;

    while(1)
    {
        /* Update the current touch. */
        touch_updateCurrentTouch(&g_sTouchContext);

        /* Check if we need to switch modes. */
        if(g_sTouchContext.touch)
        {
            if(GrRectContainsPoint(&buttonMathModeFloatBoarder,
                                   g_sTouchContext.x, g_sTouchContext.y))
            {
                if(g_ui16MathMode != DEMO_MODE_FLOAT)
                {
                    /* Set the math mode. */
                    g_ui16MathMode = DEMO_MODE_FLOAT;

                    /* Update the GUI buttons. */
                    button_drawButton(&buttonMathModeFloatBoarder,
                                      ClrDarkSlateGray,
                                      &buttonMathModeFloatFill, ClrRed,
                                      "Float");
                    button_drawButton(&buttonMathModeIQmathBoarder,
                                      ClrDarkSlateGray,
                                      &buttonMathModeIQmathFill, ClrDarkRed,
                                      "IQmathLib");
                }
            }
            else if(GrRectContainsPoint(&buttonMathModeIQmathBoarder,
                                        g_sTouchContext.x, g_sTouchContext.y))
            {
                if(g_ui16MathMode != DEMO_MODE_FIXED)
                {
                    /* Set the math mode. */
                    g_ui16MathMode = DEMO_MODE_FIXED;

                    /* Update the GUI buttons. */
                    button_drawButton(&buttonMathModeFloatBoarder,
                                      ClrDarkSlateGray,
                                      &buttonMathModeFloatFill, ClrDarkRed,
                                      "Float");
                    button_drawButton(&buttonMathModeIQmathBoarder,
                                      ClrDarkSlateGray,
                                      &buttonMathModeIQmathFill, ClrRed,
                                      "IQmathLib");
                }
            }
            else if(GrRectContainsPoint(&buttonShapeCubeFill, g_sTouchContext.x,
                                        g_sTouchContext.y))
            {
                if(g_ui16Shape != DEMO_SHAPE_CUBE)
                {
                    /* Set the shape. */
                    g_ui16Shape = DEMO_SHAPE_CUBE;

                    /* Update the GUI buttons. */
                    button_drawButton(&buttonShapeCubeBoarder, ClrDarkSlateGray,
                                      &buttonShapeCubeFill, ClrBlue, "Cube");
                    button_drawButton(&buttonShapeDodecahedronBoarder,
                                      ClrDarkSlateGray,
                                      &buttonShapeDodecahedronFill, ClrDarkBlue,
                                      "Dodecahedron");
                    button_drawButton(&buttonShapeHypercubeBoarder,
                                      ClrDarkSlateGray,
                                      &buttonShapeHypercubeFill, ClrDarkBlue,
                                      "Hypercube");
                }
            }
            else if(GrRectContainsPoint(&buttonShapeDodecahedronFill,
                                        g_sTouchContext.x, g_sTouchContext.y))
            {
                if(g_ui16Shape != DEMO_SHAPE_DODECAHEDRON)
                {
                    /* Set the shape. */
                    g_ui16Shape = DEMO_SHAPE_DODECAHEDRON;

                    /* Update the GUI buttons. */
                    button_drawButton(&buttonShapeCubeBoarder, ClrDarkSlateGray,
                                      &buttonShapeCubeFill, ClrDarkBlue,
                                      "Cube");
                    button_drawButton(&buttonShapeDodecahedronBoarder,
                                      ClrDarkSlateGray,
                                      &buttonShapeDodecahedronFill, ClrBlue,
                                      "Dodecahedron");
                    button_drawButton(&buttonShapeHypercubeBoarder,
                                      ClrDarkSlateGray,
                                      &buttonShapeHypercubeFill, ClrDarkBlue,
                                      "Hypercube");
                }
            }
            else if(GrRectContainsPoint(&buttonShapeHypercubeFill,
                                        g_sTouchContext.x, g_sTouchContext.y))
            {
                if(g_ui16Shape != DEMO_SHAPE_HYPERCUBE)
                {
                    /* Set the shape. */
                    g_ui16Shape = DEMO_SHAPE_HYPERCUBE;

                    /* Update the GUI buttons. */
                    button_drawButton(&buttonShapeCubeBoarder, ClrDarkSlateGray,
                                      &buttonShapeCubeFill, ClrDarkBlue,
                                      "Cube");
                    button_drawButton(&buttonShapeDodecahedronBoarder,
                                      ClrDarkSlateGray,
                                      &buttonShapeDodecahedronFill, ClrDarkBlue,
                                      "Dodecahedron");
                    button_drawButton(&buttonShapeHypercubeBoarder,
                                      ClrDarkSlateGray,
                                      &buttonShapeHypercubeFill, ClrBlue,
                                      "Hypercube");
                }
            }
        }

        /* Update display times every 8 frames. */
        if(ui16FrameCount >= 8)
        {
            /* Calculate new graphics time. */
            iq20GraphicsTime =
                _IQ20div((g_i32GraphicsCycles * 1000), (MCLK_FREQUENCY / 64));
            iq20GraphicsTime = _IQdiv8(iq20GraphicsTime);

            /* Calculate new display time. */
            iq20DisplayTime =
                _IQ20div((g_i32DisplayCycles * 1000), (MCLK_FREQUENCY / 64));
            iq20DisplayTime = _IQdiv8(iq20DisplayTime);

            /* Clear the previous times. */
            GrContextForegroundSet(&g_sContext, ClrBlack);
            GrRectFill(&g_sContext, &clearDisplayTimes);

            /* Change font, display the new times and restore the default font. */
            GrContextForegroundSet(&g_sContext, ClrWhite);
            GrContextFontSet(&g_sContext, &g_sFontFixed6x8);
            _IQ20toa((char *)cDisplayBuffer, "%2.2f", iq20GraphicsTime);
            GrStringDraw(&g_sContext, cDisplayBuffer, AUTO_STRING_LENGTH, 118,
                         204, TRANSPARENT_TEXT);
            _IQ20toa((char *)cDisplayBuffer, "%2.2f", iq20DisplayTime);
            GrStringDraw(&g_sContext, cDisplayBuffer, AUTO_STRING_LENGTH, 118,
                         216, TRANSPARENT_TEXT);
            if((iq20GraphicsTime + iq20DisplayTime) >= _IQ20(100))
            {
                _IQ20toa((char *)cDisplayBuffer, "%3.2f",
                         (iq20GraphicsTime + iq20DisplayTime));
                GrStringDraw(&g_sContext, cDisplayBuffer, AUTO_STRING_LENGTH,
                             112, 228, TRANSPARENT_TEXT);
            }
            else
            {
                _IQ20toa((char *)cDisplayBuffer, "%2.2f",
                         (iq20GraphicsTime + iq20DisplayTime));
                GrStringDraw(&g_sContext, cDisplayBuffer, AUTO_STRING_LENGTH,
                             118, 228, TRANSPARENT_TEXT);
            }
            GrContextFontSet(&g_sContext, &g_sFontCmss24);

            /* Reset counters. */
            ui16FrameCount = 0;
            g_i32GraphicsCycles = 0;
            g_i32DisplayCycles = 0;
        }

        demo_updateFrame(g_ui16MathMode, g_ui16Shape);

        /* Increment frame count */
        ui16FrameCount++;
    }
}

#if defined(__MSP430F5529__)

void boardInit(void)
{
    /* Set all GPIO to output low to minimize current draw. */
    GPIO_setAsOutputPin(GPIO_PORT_PA, 0xffff);
    GPIO_setAsOutputPin(GPIO_PORT_PB, 0xffff);
    GPIO_setAsOutputPin(GPIO_PORT_PC, 0xffff);
    GPIO_setAsOutputPin(GPIO_PORT_PD, 0xffff);
    GPIO_setAsOutputPin(GPIO_PORT_PJ, 0xffff);
    GPIO_setOutputLowOnPin(GPIO_PORT_PA, 0xffff);
    GPIO_setOutputLowOnPin(GPIO_PORT_PB, 0xffff);
    GPIO_setOutputLowOnPin(GPIO_PORT_PC, 0xffff);
    GPIO_setOutputLowOnPin(GPIO_PORT_PD, 0xffff);
    GPIO_setOutputLowOnPin(GPIO_PORT_PJ, 0xffff);

    return;
}

void clockInit(void)
{
    /* Set VCORE to 3.  */
    PMM_setVCore(PMM_CORE_LEVEL_3);

    /* Set MCLK = SMCLK = DCO. */
    UCS_clockSignalInit(UCS_MCLK, UCS_DCOCLK_SELECT, UCS_CLOCK_DIVIDER_1);
    UCS_clockSignalInit(UCS_SMCLK, UCS_DCOCLK_SELECT, UCS_CLOCK_DIVIDER_1);

    /* Set ACLK = REFO */
    UCS_clockSignalInit(UCS_ACLK, UCS_REFOCLK_SELECT, UCS_CLOCK_DIVIDER_1);

    /* Initialize DCO clock. */
    UCS_clockSignalInit(UCS_FLLREF, UCS_REFOCLK_SELECT, UCS_CLOCK_DIVIDER_1);
#if (MCLK_FREQUENCY == 25000000)
    UCS_initFLLSettle(25000, 763);
#elif (MCLK_FREQUENCY == 24000000)
    UCS_initFLLSettle(24000, 732);
#elif (MCLK_FREQUENCY == 16000000)
    UCS_initFLLSettle(16000, 488);
#elif (MCLK_FREQUENCY == 8000000)
    UCS_initFLLSettle(8000, 244);
#else
#error "MCLK_FREQUENCY not implemented for clockInit function."
#endif

    return;
}

#elif defined(__MSP430FR5969__)

void boardInit(void)
{
    /* Set all GPIO to output low to minimize current draw. */
    GPIO_setAsOutputPin(GPIO_PORT_PA, 0xffff);
    GPIO_setAsOutputPin(GPIO_PORT_PB, 0xffff);
    GPIO_setAsOutputPin(GPIO_PORT_PJ, 0xffff);
    GPIO_setOutputLowOnPin(GPIO_PORT_PA, 0xffff);
    GPIO_setOutputLowOnPin(GPIO_PORT_PB, 0xffff);
    GPIO_setOutputLowOnPin(GPIO_PORT_PJ, 0xffff);

    /* Disable the GPIO power-on default high-impedance mode. */
    PMM_unlockLPM5();

    return;
}

void clockInit(void)
{
#if (MCLK_FREQUENCY == 8000000)
    //Set DCO Frequency to 8MHz
    CS_setDCOFreq(CS_DCORSEL_1, CS_DCOFSEL_3);
#else
#error "MCLK_FREQUENCY not implemented for clockInit function."
#endif

    //configure MCLK, SMCLK to be source by DCOCLK
    CS_clockSignalInit(CS_SMCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_1);
    CS_clockSignalInit(CS_MCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_1);

    return;
}

#endif

void timerInit()
{
    /* Configure timer A to count cycles/64 */
    TIMER_A_configureUpMode(
        TIMER_A1_BASE,TIMER_A_CLOCKSOURCE_SMCLK,
        TIMER_A_CLOCKSOURCE_DIVIDER_64,
        UINT16_MAX,
        TIMER_A_TAIE_INTERRUPT_DISABLE,
        TIMER_A_CCIE_CCR0_INTERRUPT_DISABLE,
        TIMER_A_SKIP_CLEAR);
}

#if defined (__TI_COMPILER_VERSION__)
int16_t _system_pre_init(void)
{
    /* Stop WDT */
    WDT_A_hold(WDT_A_BASE);

    return(1);
}

#elif defined(__IAR_SYSTEMS_ICC__)
int16_t __low_level_init(void)
{
    /* Stop WDT */
    WDT_A_hold(WDT_A_BASE);

    return(1);
}

#else
#error "Compiler not supported!"
#endif
