/* --COPYRIGHT--,BSD
 * Copyright (c) 2015, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
#include <stdbool.h>
#include <stdint.h>

#include <msp430.h>

#include "driverlib.h"

#include "grlib/Library/grlib.h"
#include "grlib/LcdDriver/kitronix320x240x16_ssd2119_spi.h"

#include "QmathLib.h"
#include "IQmathLib.h"

#include "analog.h"
#include "touch.h"

/*
 * Graphic library context
 */
extern tContext g_sContext;

#if defined(__TI_COMPILER_VERSION__)
#pragma LOCATION(touch_calibrationData, 0x1980);
#pragma NOINIT(touch_calibrationData);
static touch_calibration touch_calibrationData;
#elif defined(__IAR_SYSTEMS_ICC__)
#pragma location=0x1980
__no_init static touch_calibration touch_calibrationData;
#else
#error "Compiler not supported!"
#endif

void touch_initInterface(void)
{
    /* Initialize analog interface. */
    analog_init();

    /* Configure SW1 for input. */
    GPIO_setAsInputPinWithPullUpResistor(TOUCH_SW1_PORT, TOUCH_SW1_PIN);

    /* Check if the screen has already been calibrated. */
    if(touch_calibrationData.key == TOUCH_CALIBRATION_KEY)
    {
        /* Return if the user is not manually requesting calibration. */
        if(GPIO_getInputPinValue(TOUCH_SW1_PORT, TOUCH_SW1_PIN))
        {
            return;
        }
    }

    /* Wait for SW1 to be released. */
    while(!GPIO_getInputPinValue(TOUCH_SW1_PORT, TOUCH_SW1_PIN))
    {
        ;
    }

    /* Calibrate the touch screen. */
    touch_calibrate();
}

void touch_updateCurrentTouch(touch_context *data)
{
    uint16_t ui16TouchX;
    uint16_t ui16TouchY;
    _q12 q12Ratio;

    /* Sample new X and Y coordinates if touch was detected. */
    if(analog_detectedTouch())
    {
        data->touch = true;

        /* Delay ~1ms */
        __delay_cycles(8000);

        /* Sample the X analog axis. */
        ui16TouchX = analog_sampleX();

        /* Map the analog reading to the display coordinates. */
        q12Ratio =
            _Q12div((ui16TouchX - touch_calibrationData.xMin),
                    (touch_calibrationData.xMax - touch_calibrationData.xMin));
        q12Ratio = _Qsat(q12Ratio, _Q12(1.0), _Q12(0));
        data->x = _Q12mpy(q12Ratio, LCD_HORIZONTAL_MAX);

        /* Sample the Y analog axis. */
        ui16TouchY = analog_sampleY();

        /* Map the ADC reading to the display coordinates. */
        q12Ratio =
            _Q12div((ui16TouchY - touch_calibrationData.yMin),
                    (touch_calibrationData.yMax - touch_calibrationData.yMin));
        q12Ratio = _Qsat(q12Ratio, _Q12(1.0), _Q12(0));
        data->y = _Q12mpy(q12Ratio, LCD_VERTICAL_MAX);
    }
    else
    {
        data->touch = false;
        data->x = 0xffff;
        data->y = 0xffff;
    }
}

static void touch_calibrateCircle(uint16_t *xSum,
                                  uint16_t *ySum,
                                  uint16_t xPos,
                                  uint16_t yPos)
{
    /* Draw a red circle for the user to touch. */
    GrContextForegroundSet(&g_sContext, ClrRed);
    GrCircleFill(&g_sContext, xPos, yPos, TOUCH_CALIBRATION_RADIUS);

    /* Wait for a tocuh to be detected and wait ~4ms. */
    while(!analog_detectedTouch())
    {
        ;
    }
    __delay_cycles(100000);

    /* Sample the X and Y measurements of the touch screen. */
    *xSum += analog_sampleX();
    *ySum += analog_sampleY();

    /* Wait for the touch to stop and wait ~4ms. */
    while(analog_detectedTouch())
    {
        ;
    }
    __delay_cycles(100000);

    /* Clear the drawn circle. */
    GrContextForegroundSet(&g_sContext, ClrBlack);
    GrCircleFill(&g_sContext, xPos, yPos, TOUCH_CALIBRATION_RADIUS);

    return;
}

void touch_calibrate(void)
{
    uint16_t center;
    uint16_t offset;
    touch_calibration calData;

    /* Clear existing calibration data. */
#if defined(__MSP430F5529__)
    FLASH_unlockInfoA();
    FLASH_segmentErase((uint8_t *)&touch_calibrationData);
#elif defined(__MSP430FR5969__)
    FRAM_memoryFill32(0, (uint32_t *)&touch_calibrationData,
                      sizeof(touch_calibrationData) / 4);
#endif

    /* Zero out local copies of calibration data. */
    calData.xMin = 0;
    calData.xMax = 0;
    calData.yMin = 0;
    calData.yMax = 0;

    /* Display Instructions to the screen. */
    GrContextForegroundSet(&g_sContext, ClrWhite);
    GrStringDrawCentered(&g_sContext,
                         "TOUCH DOT TO CALIBRATE",
                         AUTO_STRING_LENGTH,
                         LCD_HORIZONTAL_MAX / 2,
                         LCD_VERTICAL_MAX / 2,
                         TRANSPARENT_TEXT);

    /* Top left corner. */
    touch_calibrateCircle(
        &calData.xMin,
        &calData.yMin,
        TOUCH_CALIBRATION_RADIUS,
        TOUCH_CALIBRATION_RADIUS);

    /* Top right corner. */
    touch_calibrateCircle(
        &calData.xMax,
        &calData.yMin,
        (LCD_HORIZONTAL_MAX - 1) - TOUCH_CALIBRATION_RADIUS,
        TOUCH_CALIBRATION_RADIUS);

    /* Bottom left corner. */
    touch_calibrateCircle(
        &calData.xMin,
        &calData.yMax,
        TOUCH_CALIBRATION_RADIUS,
        (LCD_VERTICAL_MAX - 1) - TOUCH_CALIBRATION_RADIUS);

    /* Bottom right corner. */
    touch_calibrateCircle(
        &calData.xMax,
        &calData.yMax,
        (LCD_HORIZONTAL_MAX - 1) - TOUCH_CALIBRATION_RADIUS,
        (LCD_VERTICAL_MAX - 1) - TOUCH_CALIBRATION_RADIUS);

    /* Compensate for the radius offset to caluculate final X calibration values. */
    center = calData.xMin + calData.xMax;
    offset = center - (calData.xMin * 2);
    offset =
        _Q14mpy(offset,
                (_q14)(_IQ14(LCD_HORIZONTAL_MAX /
                             2) /
                       (LCD_HORIZONTAL_MAX / 2 - TOUCH_CALIBRATION_RADIUS)));
    calData.xMin = _Qdiv4(center - offset);
    calData.xMax = _Qdiv4(center + offset);

    /* Compensate for the radius offset to caluculate final Y calibration values. */
    center = calData.yMin + calData.yMax;
    offset = center - (calData.yMin * 2);
    offset =
        _Q14mpy(offset,
                (_q14)(_IQ14(LCD_VERTICAL_MAX /
                             2) /
                       (LCD_VERTICAL_MAX / 2 - TOUCH_CALIBRATION_RADIUS)));
    calData.yMin = _Qdiv4(center - offset);
    calData.yMax = _Qdiv4(center + offset);

    /* Write the calibration key to signal the values have been saved. */
    calData.key = TOUCH_CALIBRATION_KEY;

    /* Save the calibration data. */
#if defined(__MSP430F5529__)
    FLASH_write32((uint32_t *)&calData, (uint32_t *)&touch_calibrationData,
                  sizeof(touch_calibrationData) / 4);
    FLASH_lockInfoA();
#elif defined(__MSP430FR5969__)
    FRAM_write32((uint32_t *)&calData, (uint32_t *)&touch_calibrationData,
                 sizeof(touch_calibrationData) / 4);
#endif

    GrClearDisplay(&g_sContext);
}
